@@ -4,6 +4,7 @@ import urllib
 from cached_property import cached_property
 import requests
 
+from eircode.logging import logger
 from eircode.eircode import Eircode
 from eircode.constants import (
     IDENTITY_URL_PATH,
@@ -40,6 +41,10 @@ class Address():
         :kwargs display_name (optional): The display name found on eircode.ie
         :kwargs link (optional): The link that was used to get the info
         :kwargs eircode (optional): The eircode string
+        :kwargs proxy (optional): To proxy requests to the eircode api or not
+        :kwargs throw_ex (optional): To throw exceptions or gracefully fail
+        :kwargs reverse (optional): True if the input is a eircode
+            rather than an address
         '''
         self.input_address = address
 
@@ -56,13 +61,20 @@ class Address():
             )
 
     def set(self, throw_ex=True, reverse=False):
+        '''
 
+        :kwargs throw_ex (optional): To throw exceptions or gracefully fail
+        :kwargs reverse (optional): True if the input is a eircode
+            rather than an address
+        '''
         if self.proxy:
 
             try:
                 proxy.setup()
             except:
-                raise Exception('Could not set up proxy cli. Go to here for details: https://github.com/Ge0rg3/requests-ip-rotator')
+                raise Exception(
+                    'Could not set up proxy cli. Go to here for details: https://github.com/Ge0rg3/requests-ip-rotator'
+                )
 
             identity_response = proxy.get(IDENTITY_URL_PATH).json()
 
@@ -74,9 +86,15 @@ class Address():
                 'clientVersion': None
             }
 
-            finder_response = proxy.get(
-                EIRCODE_FINDER_URL_PATH + '?' + urllib.parse.urlencode(params)
-            )
+            try:
+                finder_response = proxy.get(
+                    EIRCODE_FINDER_URL_PATH + '?' + urllib.parse.urlencode(params)
+                )
+            except ValueError as ex:
+                logger.error(
+                    'Cannot search: %s' % (ex,)
+                )
+                return
 
         else:
             identity_response = requests.get(IDENTITY_URL_PATH).json()
@@ -88,10 +106,16 @@ class Address():
                 'clientVersion': None
             }
 
-            finder_response = requests.get(
-                EIRCODE_FINDER_URL_PATH,
-                params=params
-            )
+            try:
+                finder_response = requests.get(
+                    EIRCODE_FINDER_URL_PATH,
+                    params=params
+                )
+            except ValueError as ex:
+                logger.error(
+                    'Cannot search: %s' % (ex,)
+                )
+                return
 
         finder_response = finder_response.json()
 
@@ -109,7 +133,9 @@ class Address():
             if throw_ex:
                 raise Exception(finder_response['error']['text'])
             else:
-                print(f'Cannot search: {finder_response["error"]["text"]}')
+                logger.error(
+                    f'Cannot search: {finder_response["error"]["text"]}'
+                )
                 return
 
         options = finder_response['options']
@@ -127,14 +153,18 @@ class Address():
                     )
                 )
 
-            for address in addresses.ordered_best_fit(self.input_address):
+            ordered_best = addresses.ordered_best_fit(self.input_address)
+            if ordered_best:
+                address = ordered_best[0][1]
                 try:
-                    self.display_name = address[1].eircode_data['display_name']
-                    self._eircode = address[1].eircode_data['eircode']
-                except ValueError:
-                    pass
+                    self.display_name = address.eircode_data['display_name']
+                    self._eircode = address.eircode_data['eircode']
+                except ValueError as ex:
+                    logger.error('Not setting eircode values: %s' % (ex,))
         else:
-            print('TODO: use the autocomplete to see if we can get something')
+            logger.warning(
+                'TODO: use the autocomplete to see if we can get something'
+            )
 
     def match_score(self, name):
         if isinstance(name, Address):
@@ -160,11 +190,16 @@ class Address():
     @cached_property
     def eircode_data(self):
         if self.proxy:
-            data = proxy_cli.get(self.link).json()
+            data = proxy.get(self.link).json()
         else:
             data = requests.get(self.link).json()
 
-        data = requests.get(self.link).json()
+        if data.get('error', {}).get('code', None) == 403:
+            raise ValueError(data['error']['text'])
+
+        if 'options' not in data:
+            raise ValueError('Could not get options from data: %s' % (data,))
+
         if data['options']:
             addresses = Addresses()
             for option in data['options']:
@@ -178,22 +213,28 @@ class Address():
                     )
                 )
 
-            for address in addresses.ordered_best_fit(self.display_name):
+            ordered_best = addresses.ordered_best_fit(self.display_name)
+            if ordered_best:
+                address = ordered_best[0][1]
                 try:
-                    if isinstance(address[1].eircode_data, str):
+                    if isinstance(address.eircode_data, str):
                         return {
-                            'eircode': address[1].eircode_data,
-                            'display_name': address[1].display_name
+                            'eircode': address.eircode_data,
+                            'display_name': address.display_name
                         }
                     else:
-                        return address[1].eircode_data
+                        return address.eircode_data
                 except ValueError:
-                    pass
+                    raise ValueError(
+                        'Best eircode option is not good, can try more options'
+                    )
         else:
-            if data['result']['text'] == 'IncompleteAddressEntered':
-                raise ValueError('Must go back')
-            elif data['result']['text'] == 'NonUniqueAddress':
-                raise ValueError('Must go back')
+            if data['result']['text'] in {
+                'IncompleteAddressEntered',
+                'NonUniqueAddress',
+                'PostcodeNotAvailable'
+            }:
+                raise ValueError(data['result']['text'])
             else:
                 return {
                     'eircode': data['postcode'],
